//OUT widgets.out
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.Scanner;

/**
 * Homework 3 Final sample solution
 * 
 * CS 46B Introduction to Data Structures
 * San Jose State University
 * Summer 2015
 * Instructor: Ron Mak
 * 
 * @author rmak
 */
public class Widgets
{
    private static final String INPUT_FILE_NAME = "widgets.csv";
    private static final String REPORT_NAME     = "widgets.out";
    
    private Scanner     in;
    private PrintWriter out;
    
    // Column headers
    private String stateHdr, plantHdr, deptHdr, empIdHdr, nameHdr, countHdr;
    
    // Current and previous codes
    private int stateCode, plantCode, deptCode;
    private int prevState = 0, prevPlant = 0, prevDept = 0;
    
    // Totals
    private int grandTotal = 0, stateTotal = 0, plantTotal = 0, deptTotal = 0;
    
    /**
     * Read the column headers from the first input line and print them.
     */
    private void doColumnHeaders()
    {
        String headerRecord = in.nextLine();          // read the first line
        Scanner headers = new Scanner(headerRecord);  // string scanner
        headers.useDelimiter(",");
        
        // Scan the first input line for the column headers.
        stateHdr = headers.next();
        plantHdr = headers.next();
        deptHdr  = headers.next();
        empIdHdr = headers.next();
        nameHdr  = headers.next();
        countHdr = headers.next();
        
        // Print the headers.
        out.printf("%s %s %s %s %s %s\n\n",
                   stateHdr, plantHdr, deptHdr, 
                   empIdHdr, countHdr, nameHdr);
        
        headers.close();
    }
    
    /**
     * Compute and print the lines of the detail report.
     */
    private void doDetailRecords()
    {
        // Values read from a detail record.
        int id, count;
        String name;
        
        boolean firstDetailRecord = true;
        boolean printBlankLine = false;
        
        // Loop over the input detail records.
        while (in.hasNextLine()) {
            
            // Read a detail record into a string and create a
            // string scanner to read the comma-separated values.
            String detailRecord = in.nextLine();
            Scanner detail = new Scanner(detailRecord);
            detail.useDelimiter(",");

            // Read each field of a detail record.
            stateCode = detail.nextInt();
            plantCode = detail.nextInt();
            deptCode  = detail.nextInt();
            id        = detail.nextInt();
            name      = detail.next();
            count     = detail.nextInt();
            
            if (!firstDetailRecord) {
                
                // State code changed?
                if (stateCode != prevState) {
                    out.println();
                    printStateTotal();
                    printBlankLine = true;
                }
                
                // Plant code changed?
                else if (plantCode != prevPlant) {
                    out.println();
                    printPlantTotal();
                    printBlankLine = true;
                }
                
                // Department code changed?
                else if (deptCode != prevDept) {
                    out.println();
                    printDeptTotal();
                    printBlankLine = true;
                }
            }
            
            // Print a blank line after printing totals.
            if (printBlankLine) {
                out.println();
                printBlankLine = false;
            }
            
            // Set the previous codes from the line just read.
            prevDept  = deptCode;
            prevPlant = plantCode;
            prevState = stateCode;
            
            // Update the totals.
            deptTotal  += count;
            plantTotal += count;
            stateTotal += count;
            grandTotal += count;
            
            // Print a detail line.
            out.printf("%5d %5d %4d %5d %5d %s\n",
                       stateCode, plantCode, deptCode, id, count, name);
            
            detail.close();
            firstDetailRecord = false;
        }
        
        // All detail records read. Print the final state total.
        out.println();
        printStateTotal();
        
        // Print the grand total.
        out.println();
        printGrandTotal();
    }
    
    /**
     * Print a department total.
     */
    private void printDeptTotal()
    {
        out.printf("%28d TOTAL FOR DEPT  %2d *\n", 
                   deptTotal, prevDept);
        deptTotal = 0;
    }
    
    /**
     * First print a department total.
     * Then print a plant total.
     */
    private void printPlantTotal()
    {
        printDeptTotal();
        out.printf("%28d TOTAL FOR PLANT %2d **\n", 
                   plantTotal, prevPlant);
        plantTotal = 0;
    }
    
    /**
     * First print a plant total.
     * Then print a state total.
     */
    private void printStateTotal()
    {
        printPlantTotal();
        out.printf("%28d TOTAL FOR STATE %2d ***\n", 
                   stateTotal, prevState);
        stateTotal = 0;
    }
    
    /**
     * Print the grand total.
     */
    private void printGrandTotal()
    {
        out.printf("%28d GRAND TOTAL        ****\n", 
                   grandTotal);
    }
    
    /**
     * Generate a widget detail report.
     */
    private void generateReport()
    {
        doColumnHeaders();
        doDetailRecords();
    }
    
    /**
     * Initialize the input and output before generating a report.
     */
    private void initialize(String inputFileName, String outputFileName)
    {
        try {
            in  = new Scanner(new File(inputFileName));
            out = new PrintWriter(outputFileName);
            
            generateReport();
        }
        catch (FileNotFoundException ex) {
            System.out.println("*** ERROR: File not found: " + inputFileName);
        }
        finally {
            if (in  != null) in.close();
            if (out != null) out.close();
        }
    }
    
    /**
     * The main.
     * @param args the command-line parameters
     */
    public static void main(String args[])
    {
        Widgets widgets = new Widgets();
        widgets.initialize(INPUT_FILE_NAME, REPORT_NAME);
    }
}
